@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use parent qw(Exporter);
 
-our $VERSION = '0.010';
+our $VERSION = '0.011';
 
 =head1 NAME
 
@@ -12,7 +12,7 @@ Tickit::DSL - domain-specific language for Tickit terminal apps
 
 =head1 VERSION
 
-version 0.010
+Version 0.011
 
 =head1 SYNOPSIS
 
@@ -25,7 +25,8 @@ version 0.010
 =head1 DESCRIPTION
 
 WARNING: This is an early version, has an experimental API, and is
-subject to change in future. Please get in contact and/or wait for 1.0 if you want something stable.
+subject to change in future. Please get in contact and/or wait for
+1.0 if you want something stable.
 
 Provides a simplified interface for writing Tickit applications. This is
 mainly intended for prototyping:
@@ -84,6 +85,7 @@ use Tickit::Widget::CheckButton;
 use Tickit::Widget::Decoration;
 use Tickit::Widget::Entry;
 use Tickit::Widget::Frame;
+use Tickit::Widget::FloatBox;
 use Tickit::Widget::GridBox;
 use Tickit::Widget::HBox;
 use Tickit::Widget::HSplit;
@@ -96,6 +98,7 @@ use Tickit::Widget::Progressbar;
 use Tickit::Widget::RadioButton;
 use Tickit::Widget::Scroller;
 use Tickit::Widget::Scroller::Item::Text;
+use Tickit::Widget::Scroller::Item::RichText;
 use Tickit::Widget::ScrollBox;
 use Tickit::Widget::SegmentDisplay;
 use Tickit::Widget::SparkLine;
@@ -125,15 +128,15 @@ our @EXPORT = our @EXPORT_OK = qw(
 	widget customwidget
 	add_widgets
 	gridbox gridrow vbox hbox vsplit hsplit relative pane frame
+	floatbox float
 	static entry checkbox button
 	radiogroup radiobutton
-	scroller scroller_text scrollbox
+	scroller scroller_text scroller_richtext scrollbox
 	tabbed
 	tree table
 	placeholder placegrid decoration
 	statusbar
 	menubar submenu menuitem menuspacer
-	monthview
 );
 
 =head1 METHODS
@@ -564,7 +567,7 @@ sub scrollbox(&@) {
 		local $PARENT = 'Tickit::Widget::ScrollBox';
 		local @PENDING_CHILD;
 		$code->();
-		
+
 		Tickit::Widget::ScrollBox->new(
 			child => $PENDING_CHILD[0],
 			%args
@@ -614,6 +617,23 @@ sub scroller_text {
 	apply_widget($w);
 }
 
+=head2 scroller_richtext
+
+A text item, expects to be added to a L</scroller>. The item itself should be
+a L<String::Tagged> instance, like this:
+
+ my $str = String::Tagged->new( "An important message" );
+ $str->apply_tag( 3, 9, b => 1 );
+ scroller_richtext $str;
+
+
+=cut
+
+sub scroller_richtext {
+	my $w = Tickit::Widget::Scroller::Item::RichText->new(shift);
+	apply_widget($w);
+}
+
 =head1 FUNCTIONS - Miscellaneous container
 
 These act as containers.
@@ -652,6 +672,78 @@ sub tabbed(&@) {
 	apply_widget($w);
 }
 
+=head2 floatbox
+
+A container which normally has no visible effect, but provides the ability to contain L</float>s.
+These are floating windows which can be located anywhere within the container, usually for the purpose
+of providing dynamic windows such as popups and dropdowns.
+
+ floatbox {
+  vbox {
+   button {
+    float {
+     static 'this is a float'
+	} lines => 3, top => -1, left => '-50%';
+   } 'Show';
+  }
+ }
+
+=cut
+
+sub floatbox(&@) {
+	my ($code, %args) = @_;
+	my %parent_args = map {; $_ => delete $args{'parent:' . $_} } map /^parent:(.*)/ ? $1 : (), keys %args;
+	my $w = Tickit::Widget::FloatBox->new(%args);
+	{
+		local $PARENT = $w;
+		$code->($w);
+	}
+	local @WIDGET_ARGS = (@WIDGET_ARGS, %parent_args);
+	apply_widget($w);
+}
+
+=head2 float
+
+A L</float> provides a floating window within a L</floatbox> container - note that the L</floatbox>
+does not need to be an immediate parent.
+
+ floatbox {
+  vbox {
+   button {
+    float {
+     static 'this is a float'
+	} lines => 3, top => -1, left => '-50%';
+   } 'Show';
+  }
+ }
+
+=cut
+
+sub float(&@) {
+	my ($code, %args) = @_;
+	my %parent_args = map {; $_ => delete $args{'parent:' . $_} } map /^parent:(.*)/ ? $1 : (), keys %args;
+
+	# Work out which container to use - either the least-distant ancestor,
+	# or a specific floatbox if one was provided
+	my $floatbox = delete($args{container}) || $PARENT;
+	while($floatbox && !$floatbox->isa('Tickit::Widget::FloatBox')) {
+		$floatbox = $floatbox->parent;
+	}
+	die "No floatbox found for this float" unless $floatbox;
+
+	my $w = Tickit::Widget::VBox->new;
+	$floatbox->add_float(
+		child => $w,
+		%args
+	);
+	# The new float won't be visible yet, defer this code until the
+	# window is ready.
+	later {
+		local $PARENT = $w;
+		$code->($w);
+	};
+}
+
 =head2 statusbar
 
 A L<Tickit::Widget::Statusbar>. Not very exciting.
@@ -670,6 +762,8 @@ sub statusbar(&@) {
 	apply_widget($w);
 }
 
+=head1 FUNCTIONS - General widgets
+
 =head2 static
 
 Static text. Very simple:
@@ -784,11 +878,17 @@ second parameter is the label:
 
 sub button(&@) {
 	my $code = shift;
-	my %args = (on_click => $code, label => @_);
+	my %args = (
+		label => @_
+	);
 	my %parent_args = map {; $_ => delete $args{'parent:' . $_} } map /^parent:(.*)/ ? $1 : (), keys %args;
 	my $w = Tickit::Widget::Button->new(
 		%args
 	);
+	$w->set_on_click(sub {
+		local $PARENT = $w->parent;
+		$code->($w->parent);
+	});
 	{
 		local @WIDGET_ARGS = (@WIDGET_ARGS, %parent_args);
 		apply_widget($w);
@@ -1041,6 +1141,9 @@ sub apply_widget {
 			$PARENT->add_tab($w, @WIDGET_ARGS);
 		} elsif($PARENT->isa('Tickit::Widget::GridBox')) {
 			$PARENT->add($GRID_ROW, $GRID_COL++, $w, @WIDGET_ARGS);
+		} elsif($PARENT->isa('Tickit::Widget::FloatBox')) {
+			# Needs 0.02+ to ensure parent is set correctly
+			$PARENT->set_base_child($w);
 		} else {
 			$PARENT->add($w, @WIDGET_ARGS);
 		}
@@ -1070,6 +1173,8 @@ __END__
 
 =item * L<Tickit::Widget::Entry>
 
+=item * L<Tickit::Widget::FloatBox>
+
 =item * L<Tickit::Widget::Frame>
 
 =item * L<Tickit::Widget::GridBox>
@@ -1120,4 +1225,4 @@ Tom Molesworth <cpan@entitymodel.com>
 
 =head1 LICENSE
 
-Copyright Tom Molesworth 2012-2013. Licensed under the same terms as Perl itself.
+Copyright Tom Molesworth 2012-2014. Licensed under the same terms as Perl itself.
